<?php

declare(strict_types=1);

namespace Drupal\Tests\confirmation\Kernel;

use Drupal\confirmation\Entity\Event\ConfirmationEvent;
use Drupal\confirmation_event_test\LoggingConfirmationEventSubscriber;
use Drupal\confirmation_example\Plugin\ConfirmationBundle\ExampleConfirmationBundle;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

/**
 * Kernel tests with more dependencies.
 *
 * @group confirmation
 */
final class ExampleConfirmationWithDependenciesKernelTest extends KernelTestBase {

  protected static $modules = [
    'entity',
    'node',
    'user',
    'field',
    'confirmation',
    'confirmation_example',
    'confirmation_event_test',
  ];

  protected NodeInterface $node;

  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installSchema('node', 'node_access');
    $this->installEntitySchema('node');
    $this->installEntitySchema('confirmation');

    $this->node = Node::create([
      'type' => 'article',
      'title' => 'ArticleToConfirm',
    ]);
    $this->node->save();
  }

  public function testDefaultExpiration() {
    $future1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $this->assertSame(ExampleConfirmationBundle::getExpirationInterval(), $future1->getExpirationTime() - \Drupal::time()->getRequestTime());
  }

  public function testPurge() {
    $future1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $future1->save();

    $future2 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $future2->save();

    $past1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node)
      ->setExpirationInterval(-1);
    $past1->save();

    $this->assertSame(TRUE, $past1->isExpired());
    $this->assertSame(FALSE, $future1->isExpired());

    $toPurge = ExampleConfirmationBundle::getEntityIdsToPurge('confirmation_example', 100);
    self::assertSame([$past1->id()], array_values($toPurge));
  }

  public function testEvents(): void {
    $loggingEventSubscriber = \Drupal::service('confirmation_event_test.subscriber');
    assert($loggingEventSubscriber instanceof LoggingConfirmationEventSubscriber);
    $loggingEventSubscriber->getAndClearEvents();
    $confirmation = ExampleConfirmationBundle::create();
    $confirmation->setNode($this->node);
    $confirmation->save();
    $confirmation->save();
    $confirmation->setState(TRUE);
    $this->assertEmpty($loggingEventSubscriber->getAndClearEvents());
    $confirmation->save();
    $events = $loggingEventSubscriber->getAndClearEvents();
    $this->assertCount(2, $events);
    $this->assertSame('confirmation.confirmation_example.state_settled', $events[0][1]);
    $this->assertSame('confirmation.state_settled', $events[1][1]);
    $this->assertInstanceOf(ConfirmationEvent::class, $events[0][0]);
    $this->assertInstanceOf(ConfirmationEvent::class, $events[1][0]);
  }

}
